<?php declare(strict_types=1);

/*
 *     Pushbullet Notifier [sikofitt/pushbullet-notifier]
 *     Copyright (C) 2017  http://blog.rewiv.com R. Eric Wheeler <sikofitt@gmail.com>
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Pushbullet\Console\Command;

use Pushbullet\Exceptions\NotFoundException;
use Pushbullet\Pushbullet;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Exception\InvalidArgumentException;
use Symfony\Component\Console\Exception\LogicException;
use Symfony\Component\Console\Exception\RuntimeException;
use Symfony\Component\Console\Helper\TableCell;
use Symfony\Component\Console\Helper\TableSeparator;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Command to list available targets from a Pushbullet account.
 */
class PushbulletDevicesCommand extends Command
{
    /** @var array  */
    private $table;

    /** @var  SymfonyStyle */
    private $io;

    /** @var \Pushbullet\Pushbullet */
    private $pushbullet;

    /** @var  string */
    private $type;

    /** @var  ContainerInterface */
    private $container;

    /**
     * PushbulletDevicesCommand constructor.
     *
     * @param ContainerInterface $container
     *
     * @throws RuntimeException
     * @throws LogicException
     * @throws InvalidArgumentException
     * @throws \Symfony\Component\DependencyInjection\Exception\InvalidArgumentException
     */
    public function __construct(ContainerInterface $container)
    {
        $this->table = [];
        $this->container = $container;
        $token = $container->getParameter('pushbullet.token');

        try {
            $this->pushbullet = new Pushbullet($token);
        } catch (\Exception $e) {
            throw new RuntimeException($e->getMessage(), $e->getCode(), $e);
        }

        parent::__construct('targets');
    }

    /**
     * Configures the current command.
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this->setName('targets')
            ->setDescription('Gets a list of available pushbullet targets.')
          ->setHelp($this->getHelpText())
          ->addArgument(
            'target',
            InputArgument::OPTIONAL,
            "Which types you want to display.\n[all, devices, contacts, channels]",
            'all'
          );
    }

    /**
     * Interacts with the user.
     *
     * This method is executed before the InputDefinition is validated.
     * This means that this is the only place where the command can
     * interactively ask for values of missing required arguments.
     *
     * @param InputInterface $input An InputInterface instance
     * @param OutputInterface $output An OutputInterface instance
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function interact(InputInterface $input, OutputInterface $output): void
    {
        $this->io = new SymfonyStyle($input, $output);

        $this->type = strtolower($input->getArgument('target'));

        $availableTypes = ['all', 'devices', 'channels', 'contacts'];

        if (false === array_key_exists($this->type, array_flip($availableTypes))) {
            throw new InvalidArgumentException('display must be one of [' . implode(', ', $availableTypes) . ']');
        }
    }

    /**
     * Executes the current command.
     *
     * This method is not abstract because you can use this class
     * as a concrete class. In this case, instead of defining the
     * execute() method, you set the code to execute by passing
     * a Closure to the setCode() method.
     *
     * @param InputInterface $input An InputInterface instance
     * @param OutputInterface $output An OutputInterface instance
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @throws \Symfony\Component\Console\Exception\RuntimeException
     * @throws LogicException When this abstract method is not implemented
     * @throws \Pushbullet\Exceptions\NotFoundException
     * @throws \Pushbullet\Exceptions\InvalidTokenException
     * @throws \Pushbullet\Exceptions\ConnectionException
     *
     * @return null|int null or 0 if everything went fine, or an error code
     *
     * @see setCode()
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        $this->io->note('You can push by iden or Nick name');
        $heading = ['Nick name', 'Make', 'Model', 'iden'];

        switch ($this->type) {
            case 'devices':
                $this->io->text(' <fg=white;options=bold>Devices</>');
                $this->addDevicesSection();
                break;
            case 'channels':
                $heading = ['Name', 'Description', 'iden'];
                $this->io->text(' <fg=white;options=bold>Channels</>');
                $this->addChannelsSection();
                break;
            case 'contacts':
                $heading = ['Name', 'Email', 'iden'];
                $this->io->text(' <fg=white;options=bold>Contacts</>');
                $this->addContactsSection();
                break;
            default:
                // devices
                $this->addDevicesHeading();
                $this->addDevicesSection();
                // channels
                $this->addChannelsHeading();
                $this->addChannelsSection();
                // contacts
                $this->addContactsHeading();
                $this->addContactsSection();
                break;
        }

        if (true !== empty($this->table)) {
            $logger = $this->container->get('logger');
            $logger->debug('Creating table');
            $this->io->table($heading, $this->table);
            return 0;
        }

        $this->io->error('There were no devices found.');

        return 255;
    }

    /**
     * @return string
     *   Help text for this command.
     */
    private function getHelpText(): string
    {
        return 'This command displays your available devices, contacts, and channels.' . PHP_EOL .
                'By default this will display all of your targets.' . PHP_EOL .
                'You can list specific ones by using the argument target.' . PHP_EOL .
                '[{path}/bin/pushbullet targets contacts], for example will only display your contact targets.' . PHP_EOL;
    }

    /**
     * Sets the devices listing header.
     */
    private function addDevicesHeading(): void
    {
        $this->io->text(' <fg=white;options=bold>Devices</>');
    }

    /**
     * Sets the channels listing header.
     */
    private function addChannelsHeading(): void
    {
        $tableSeparator = new TableSeparator();
        $tableCell = new TableCell();
        $this->table[] = $tableSeparator;
        $this->table[] = [
          new TableCell('<fg=white;options=bold>Channels</>'),
          $tableCell,
          $tableCell,
          $tableCell
        ];
        $this->table[] = new TableSeparator();
        $this->table[] = [
          new TableCell('<fg=green>Name</>'),
          new TableCell('<fg=green>Description</>'),
          new TableCell(),
          new TableCell('<fg=green>Iden</>')
        ];
        $this->table[] = new TableSeparator();
    }

    /**
     * Sets the contacts listing header.
     */
    private function addContactsHeading(): void
    {
        $this->table[] = new TableSeparator();
        $this->table[] = [
          new TableCell('<fg=white;options=bold>Contacts</>'),
          new TableCell(),
          new TableCell(),
          new TableCell()
        ];
        $this->table[] = new TableSeparator();
        $this->table[] = [
          new TableCell('<fg=green>Name</>'),
          new TableCell('<fg=green>Email</>'),
          new TableCell(),
          new TableCell('<fg=green>Iden</>')
        ];
        $this->table[] = new TableSeparator();
    }

    /**
     * Adds the devices section of the table.
     *
     * @throws \Pushbullet\Exceptions\ConnectionException
     * @throws \Pushbullet\Exceptions\InvalidTokenException
     */
    private function addDevicesSection(): void
    {
        try {
            $pushbulletDevices = $this->pushbullet->getDevices();
        } catch (\Exception $e) {
            $this->io->error($e->getMessage());
            return;
        }

        foreach ($pushbulletDevices as $device) {
            $this->table[] = [
              $device->nickname,
              $device->manufacturer,
              $device->model,
              $device->iden
            ];
        }
    }

    /**
     * Adds the contacts section of the table.
     *
     * @throws \Pushbullet\Exceptions\ConnectionException
     * @throws \Pushbullet\Exceptions\InvalidTokenException
     */
    private function addContactsSection(): void
    {
        try {
            $contacts = $this->pushbullet->getContacts();
        } catch (\Exception $e) {
            $this->io->error($e->getMessage());
            return;
        }

        foreach ($contacts as $contact) {
            if ('all' === $this->type) {
                $this->table[] = [
                  $contact->name,
                  $contact->email,
                  new TableCell(),
                  $contact->iden
                ];
            } else {
                $this->table[] = [
                  $contact->name,
                  $contact->email,
                  $contact->iden,
                ];
            }
        }
    }

    /**
     * Adds the channels section of the table.
     *
     * @throws \Pushbullet\Exceptions\ConnectionException
     * @throws \Pushbullet\Exceptions\InvalidTokenException
     * @throws \Pushbullet\Exceptions\NotFoundException
     */
    private function addChannelsSection(): void
    {
        try {
            $channels = $this->pushbullet->getMyChannels();
        } catch (NotFoundException $n) {
            $this->io->error('Could not find any channels.');
            return;
        } catch (\Exception $e) {
            return;
        }

        foreach ($channels as $channel) {
            if ('all' === $this->type) {
                $this->table[] = [
                  $channel->name,
                  $channel->description,
                  new TableCell(),
                  $channel->iden
                ];
            } else {
                $this->table[] = [
                  $channel->name,
                  $channel->description,
                  $channel->iden
                ];
            }
        }
    }
}
