<?php declare(strict_types=1);

/*
 *     Pushbullet Notifier [sikofitt/pushbullet-notifier]
 *     Copyright (C) 2017  http://blog.rewiv.com R. Eric Wheeler <sikofitt@gmail.com>
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Pushbullet\Console\Command;

use Pushbullet\Push;
use Pushbullet\Pushbullet;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Exception\InvalidArgumentException;
use Symfony\Component\Console\Exception\LogicException;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Command to send notifications through Pushbullet.
 */
class PushbulletNotifierCommand extends Command
{
    /** @var \Pushbullet\Pushbullet  */
    private $pushbullet;

    /** @var string */
    private $typeFunction;

    /**
     * Constructor.
     *
     * @param $container ContainerInterface
     *
     * @throws LogicException When the command name is empty
     * @throws \Pushbullet\Exceptions\PushbulletException
     * @throws \Symfony\Component\DependencyInjection\Exception\InvalidArgumentException
     */
    public function __construct(ContainerInterface $container)
    {
        $token = $container->getParameter('pushbullet.token');
        $this->pushbullet = new Pushbullet($token);
        parent::__construct();
    }

    /**
     * Configures the current command.
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this->setName('notify')
            ->setDescription('Notifies using Pushbullet.')
            ->addArgument(
              'name',
              InputArgument::IS_ARRAY,
              'The name or ident of the device to push to.'
            )
            ->addOption(
              'device',
              'd',
              InputOption::VALUE_NONE,
              'Push to a device'
            )
            ->addOption(
              'channel',
              'c',
              InputOption::VALUE_NONE,
              'Push to a channel'
            )
            ->addOption(
              'contact',
              't',
              InputOption::VALUE_NONE,
              'Push to a contact'
            )
            ->addOption(
              'title',
              null,
              InputOption::VALUE_OPTIONAL,
              'The Title of your note.',
              'Note from ' . getenv('USER')
            )
            ->addOption(
              'message',
              'm',
              InputOption::VALUE_REQUIRED,
              'Your message.  You should quote this value.'
            )
        ;
    }

    /**
     * Interacts with the user.
     *
     * This method is executed before the InputDefinition is validated.
     * This means that this is the only place where the command can
     * interactively ask for values of missing required arguments.
     *
     * @param InputInterface $input An InputInterface instance
     * @param OutputInterface $output An OutputInterface instance
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function interact(InputInterface $input, OutputInterface $output): void
    {
        if (null === $input->getOption('message')) {
            throw new InvalidArgumentException('Message MUST be set.');
        }

        if (null === $input->getArgument('name') || true === empty($input->getArgument('name'))) {
            throw new InvalidArgumentException('Name must be set');
        }

        $device = $input->getOption('device');
        $channel = $input->getOption('channel');
        $contact = $input->getOption('contact');

        if (
            ($device && $channel && $contact) ||
            ($device && $channel) ||
            ($device && $contact) ||
            ($channel && $contact)
        ) {
            throw new InvalidArgumentException('Can only set one target type.');
        }

        if ($device) {
            $this->typeFunction = 'device';
        } elseif ($channel) {
            $this->typeFunction = 'channel';
        } elseif ($contact) {
            $this->typeFunction = 'contact';
        } else {
            throw new InvalidArgumentException('Need to set at least one type');
        }
    }

    /**
     * Executes the current command.
     *
     * This method is not abstract because you can use this class
     * as a concrete class. In this case, instead of defining the
     * execute() method, you set the code to execute by passing
     * a Closure to the setCode() method.
     *
     * @param InputInterface $input An InputInterface instance
     * @param OutputInterface $output An OutputInterface instance
     *
     * @throws LogicException When this abstract method is not implemented
     * @throws \Pushbullet\Exceptions\NotPushableException
     * @throws \Pushbullet\Exceptions\ConnectionException
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @return null|int null or 0 if everything went fine, or an error code
     *
     * @see setCode()
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        $io = new SymfonyStyle($input, $output);
        $iden = $input->getArgument('name');

        $iden = implode(' ', $iden);
        /** @var \Pushbullet\Device|\Pushbullet\Contact|\Pushbullet\Channel $type */
        $type = $this->pushbullet->{$this->typeFunction}($iden);

        $title = $input->getOption('title');
        $message = $input->getOption('message');

        $result = $type->pushNote($title, $message);

        if ($result instanceof Push) {
            $io->success('Your note has been pushed!');
            return 0;
        }
        return 255;
    }
}
